@@ -5,11 +5,17 @@ from .option import (
     Traefik2OptionsPort,
     Traefik2Options,
 )
+from .configfile import (
+    Traefik2ConfigFileOptions,
+    Traefik2ConfigFile,
+)
 
-__version__ = "0.7.4"
+__version__ = "0.8.0"
 
 __all__ = [
     'Traefik2Builder',
     'Traefik2Options',
     'Traefik2OptionsPort',
+    'Traefik2ConfigFileOptions',
+    'Traefik2ConfigFile',
 ]
